@@ -22,9 +22,11 @@ class BOTAN_DLL EMSA_X931 final : public EMSA
    {
    public:
       /**
-      * @param hash the hash object to use
+      * @param hash the hash function to use
       */
       explicit EMSA_X931(HashFunction* hash);
+
+      EMSA* clone() override { return new EMSA_X931(m_hash->clone()); }
    private:
       void update(const byte[], size_t) override;
       secure_vector<byte> raw_data() override;
