@@ -21,10 +21,8 @@ class BOTAN_DLL OAEP final : public EME
    public:
       size_t maximum_input_size(size_t) const override;
 
-      static OAEP* make(const Spec& spec);
-
       /**
-      * @param hash object to use for hashing (takes ownership)
+      * @param hash function to use for hashing (takes ownership)
       * @param P an optional label. Normally empty.
       */
       OAEP(HashFunction* hash, const std::string& P = "");
