@@ -25,7 +25,8 @@ class BOTAN_DLL TLS_PRF final : public KDF
 
       size_t kdf(byte key[], size_t key_len,
                  const byte secret[], size_t secret_len,
-                 const byte salt[], size_t salt_len) const override;
+                 const byte salt[], size_t salt_len,
+                 const byte label[], size_t label_len) const override;
 
       TLS_PRF();
    private:
@@ -45,11 +46,13 @@ class BOTAN_DLL TLS_12_PRF final : public KDF
 
       size_t kdf(byte key[], size_t key_len,
                  const byte secret[], size_t secret_len,
-                 const byte salt[], size_t salt_len) const override;
+                 const byte salt[], size_t salt_len,
+                 const byte label[], size_t label_len) const override;
 
+      /**
+      * @param mac MAC algorithm to use
+      */
       explicit TLS_12_PRF(MessageAuthenticationCode* mac) : m_mac(mac) {}
-
-      static TLS_12_PRF* make(const Spec& spec);
    private:
       std::unique_ptr<MessageAuthenticationCode> m_mac;
    };
